from typing import Final

BAD_ALIGN: Final[int]
BAD_ALPHA_CHANNEL: Final[int]
BAD_CALL_BACK: Final[int]
BAD_COI: Final[int]
BAD_DATA_PTR: Final[int]
BAD_DEPTH: Final[int]
BAD_IMAGE_SIZE: Final[int]
BAD_MODEL_OR_CH_SEQ: Final[int]
BAD_NUM_CHANNEL1U: Final[int]
BAD_NUM_CHANNELS: Final[int]
BAD_OFFSET: Final[int]
BAD_ORDER: Final[int]
BAD_ORIGIN: Final[int]
BAD_ROISIZE: Final[int]
BAD_STEP: Final[int]
BAD_TILE_SIZE: Final[int]
BadAlign: Final[int]
BadAlphaChannel: Final[int]
BadCOI: Final[int]
BadCallBack: Final[int]
BadDataPtr: Final[int]
BadDepth: Final[int]
BadImageSize: Final[int]
BadModelOrChSeq: Final[int]
BadNumChannel1U: Final[int]
BadNumChannels: Final[int]
BadOffset: Final[int]
BadOrder: Final[int]
BadOrigin: Final[int]
BadROISize: Final[int]
BadStep: Final[int]
BadTileSize: Final[int]
GPU_API_CALL_ERROR: Final[int]
GPU_NOT_SUPPORTED: Final[int]
GpuApiCallError: Final[int]
GpuNotSupported: Final[int]
HEADER_IS_NULL: Final[int]
HeaderIsNull: Final[int]
MASK_IS_TILED: Final[int]
MaskIsTiled: Final[int]
OPEN_CLAPI_CALL_ERROR: Final[int]
OPEN_CLDOUBLE_NOT_SUPPORTED: Final[int]
OPEN_CLINIT_ERROR: Final[int]
OPEN_CLNO_AMDBLAS_FFT: Final[int]
OPEN_GL_API_CALL_ERROR: Final[int]
OPEN_GL_NOT_SUPPORTED: Final[int]
OpenCLApiCallError: Final[int]
OpenCLDoubleNotSupported: Final[int]
OpenCLInitError: Final[int]
OpenCLNoAMDBlasFft: Final[int]
OpenGlApiCallError: Final[int]
OpenGlNotSupported: Final[int]
STS_ASSERT: Final[int]
STS_AUTO_TRACE: Final[int]
STS_BACK_TRACE: Final[int]
STS_BAD_ARG: Final[int]
STS_BAD_FLAG: Final[int]
STS_BAD_FUNC: Final[int]
STS_BAD_MASK: Final[int]
STS_BAD_MEM_BLOCK: Final[int]
STS_BAD_POINT: Final[int]
STS_BAD_SIZE: Final[int]
STS_DIV_BY_ZERO: Final[int]
STS_ERROR: Final[int]
STS_FILTER_OFFSET_ERR: Final[int]
STS_FILTER_STRUCT_CONTENT_ERR: Final[int]
STS_INPLACE_NOT_SUPPORTED: Final[int]
STS_INTERNAL: Final[int]
STS_KERNEL_STRUCT_CONTENT_ERR: Final[int]
STS_NOT_IMPLEMENTED: Final[int]
STS_NO_CONV: Final[int]
STS_NO_MEM: Final[int]
STS_NULL_PTR: Final[int]
STS_OBJECT_NOT_FOUND: Final[int]
STS_OK: Final[int]
STS_OUT_OF_RANGE: Final[int]
STS_PARSE_ERROR: Final[int]
STS_UNMATCHED_FORMATS: Final[int]
STS_UNMATCHED_SIZES: Final[int]
STS_UNSUPPORTED_FORMAT: Final[int]
STS_VEC_LENGTH_ERR: Final[int]
StsAssert: Final[int]
StsAutoTrace: Final[int]
StsBackTrace: Final[int]
StsBadArg: Final[int]
StsBadFlag: Final[int]
StsBadFunc: Final[int]
StsBadMask: Final[int]
StsBadMemBlock: Final[int]
StsBadPoint: Final[int]
StsBadSize: Final[int]
StsDivByZero: Final[int]
StsError: Final[int]
StsFilterOffsetErr: Final[int]
StsFilterStructContentErr: Final[int]
StsInplaceNotSupported: Final[int]
StsInternal: Final[int]
StsKernelStructContentErr: Final[int]
StsNoConv: Final[int]
StsNoMem: Final[int]
StsNotImplemented: Final[int]
StsNullPtr: Final[int]
StsObjectNotFound: Final[int]
StsOk: Final[int]
StsOutOfRange: Final[int]
StsParseError: Final[int]
StsUnmatchedFormats: Final[int]
StsUnmatchedSizes: Final[int]
StsUnsupportedFormat: Final[int]
StsVecLengthErr: Final[int]
