from typing import Final

MM_AFFINE: Final[int]
MM_HOMOGRAPHY: Final[int]
MM_RIGID: Final[int]
MM_ROTATION: Final[int]
MM_SIMILARITY: Final[int]
MM_TRANSLATION: Final[int]
MM_TRANSLATION_AND_SCALE: Final[int]
MM_UNKNOWN: Final[int]
