from typing import Any, TypeAlias

retval: TypeAlias = Any

def getIppVersion() -> retval:
    """
    .
    """

def setUseIPP(flag) -> None:
    """
    .
    """

def setUseIPP_NotExact(flag) -> None:
    """
    .
    """

def useIPP() -> retval:
    """
    proxy for hal::Cholesky
    """

def useIPP_NotExact() -> retval:
    """
    .
    """
