from typing import Final

COLOR_CAMERA_PARAMS_BOARD_SOCKET_BGR: Final[int]
COLOR_CAMERA_PARAMS_BOARD_SOCKET_RGB: Final[int]
COLOR_CAMERA_PARAMS_RESOLUTION_THE_1080_P: int
ColorCameraParams_BoardSocket_BGR: Final[int]
ColorCameraParams_BoardSocket_RGB: Final[int]
ColorCameraParams_Resolution_THE_1080_P: int
ENCODER_CONFIG_PROFILE_H264_BASELINE: Final[int]
ENCODER_CONFIG_PROFILE_H264_HIGH: Final[int]
ENCODER_CONFIG_PROFILE_H264_MAIN: Final[int]
ENCODER_CONFIG_PROFILE_H265_MAIN: Final[int]
ENCODER_CONFIG_PROFILE_MJPEG: Final[int]
ENCODER_CONFIG_RATE_CONTROL_MODE_CBR: Final[int]
ENCODER_CONFIG_RATE_CONTROL_MODE_VBR: Final[int]
EncoderConfig_Profile_H264_BASELINE: Final[int]
EncoderConfig_Profile_H264_HIGH: Final[int]
EncoderConfig_Profile_H264_MAIN: Final[int]
EncoderConfig_Profile_H265_MAIN: Final[int]
EncoderConfig_Profile_MJPEG: Final[int]
EncoderConfig_RateControlMode_CBR: Final[int]
EncoderConfig_RateControlMode_VBR: Final[int]
